@@ -1,6 +1,6 @@
 /* Mergeant
  *
- * Copyright (C) 1999 - 2005 Vivien Malerba
+ * Copyright (C) 1999 - 2006 Vivien Malerba
  * Copyright (C) 2002 - 2003 Rodrigo Moya
  *
  * Authors:
@@ -27,6 +27,7 @@
 #include <glib/gspawn.h>
 #include <gtk/gtkstatusbar.h>
 #include <gtk/gtktable.h>
+#include <glib/gi18n-lib.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomedb/libgnomedb.h>
 #include <libgnomedb/gnome-db-login-dialog.h>
@@ -34,16 +35,18 @@
 #include "workspace.h"
 #include "workspace-window.h"
 #include <libgda/gda-threader.h>
+#include "utils.h"
+
+#define PROGRESS_BAR_TIMEOUT 20
 
 typedef struct {
-	GtkWidget *window;
-	GtkWidget *workspace;
-	GtkWidget *status_bar;
+	GtkWidget      *window;
+	GtkWidget      *workspace;
+	GtkWidget      *status_bar;
 
 	/* the connection to the database */
-	GnomeDbDict    *dict;
-	GnomeDbServer  *server;
-
+	GdaDict        *dict;
+	
 	GSList         *consoles; /* list of GtkDialog for Sql consoles */
 
 	GdaTransaction *pending_transaction;
@@ -53,7 +56,7 @@ static GList         *opened_workspaces = NULL;
 static GtkWidget     *dsn_dialog = NULL;
 
 static void perform_dbms_update (WorkspaceWindowPrivate *priv);
-static void stop_dbms_update_cb (GtkWidget *dialog, gint response, GnomeDbDict *dict);
+static void stop_dbms_update_cb (GtkWidget *dialog, gint response, GdaDict *dict);
 
 static void
 destroy_private_data_cb (WorkspaceWindowPrivate *priv)
@@ -65,10 +68,9 @@ destroy_private_data_cb (WorkspaceWindowPrivate *priv)
 			gtk_widget_destroy (GTK_WIDGET (priv->consoles->data));
 
 		if (priv->dict) {
-			gnome_db_dict_save_xml (priv->dict, NULL); /* save before destruction */
+			gda_dict_save (priv->dict, NULL); /* save before destruction */
 			g_object_unref (priv->dict);
 			priv->dict = NULL;
-			priv->server = NULL;
 		}
 
 		if (priv->pending_transaction) {
@@ -84,90 +86,124 @@ destroy_private_data_cb (WorkspaceWindowPrivate *priv)
 }
 
 static void
-on_file_new_workspace (GtkAction *action, void *data)
+on_file_new_workspace (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 	create_new_workspace_window (NULL);
 }
 
 static void
-on_file_copy_workspace (GtkAction *action, void *data)
+on_file_copy_workspace (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-	WorkspaceWindowPrivate *priv;
-
-	priv = g_object_get_data (G_OBJECT (data), "Mergeant_WorkspaceWindowPrivate");
 	create_new_workspace_window (priv->dict);
 }
 
 static void
-on_file_close (GtkAction *action, void *data)
+on_file_close (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-	GtkWidget *window = data;
+	GtkWidget *window = priv->window;
 	
 	gtk_widget_destroy (window);
 }
 
 static void
-on_file_save (GtkAction *action, void *data)
+on_file_save (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-	WorkspaceWindowPrivate *priv;
 	GError *error = NULL;
 
-	priv = g_object_get_data (G_OBJECT (data), "Mergeant_WorkspaceWindowPrivate");
-
-	/* saving of GnomeDbDict */
-	if (!gnome_db_dict_save_xml (priv->dict, &error)) {
+	/* saving of GdaDict */
+	if (!gda_dict_save (priv->dict, &error)) {
 		g_warning ("Could not save: %s\n", error->message);
 		g_error_free (error);
 	}
 }
 
 static void
-on_file_exit (GtkAction *action, void *data)
+on_file_exit (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 	while (opened_workspaces) {
-		WorkspaceWindowPrivate *priv = opened_workspaces->data;
+		priv = opened_workspaces->data;
 
-		on_file_close (NULL, priv->window);
+		on_file_close (NULL, priv);
 	}
 }
 
 static void
-on_database_sync (GtkAction *action, void *data)
+on_database_sync (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-	WorkspaceWindowPrivate *priv;
-
-	priv = g_object_get_data (G_OBJECT (data), "Mergeant_WorkspaceWindowPrivate");
-
 	perform_dbms_update (priv);
-	gnome_db_dict_save_xml (priv->dict, NULL);
+	gda_dict_save (priv->dict, NULL);
 }
 
 static void
-on_database_begin (GtkAction *action, void *data)
+on_database_begin (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 }
 
 static void
-on_database_commit (GtkAction *action, void *data)
+on_database_commit (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 }
 
 static void
-on_database_rollback (GtkAction *action, void *data)
+on_database_rollback (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 }
 
+
 static void
-on_tools_data_sources (GtkAction *action, void *data)
+on_database_new (GtkAction *action, WorkspaceWindowPrivate *priv)
+{
+	GtkWidget *dlg;
+	gchar *str;
+
+	str = g_strdup_printf ("%s\n<small>%s</small>",
+			       _("Database creation requires that first a database provider be selected."),
+			       _("The 'SQLite' provider allows you to create a database stored in a single file\n"
+				 "and does not require a database server to be set up."));
+	dlg = mergeant_server_op_create_dialog (GTK_WINDOW (priv->window), GDA_SERVER_OPERATION_CREATE_DB, 
+						gda_dict_get_connection (priv->dict), NULL,
+						_("Create a database"), str, NULL, NULL);
+	g_free (str);
+	gtk_widget_show (dlg);
+}
+
+static void
+on_database_del (GtkAction *action, WorkspaceWindowPrivate *priv)
+{
+	GtkWidget *dlg;
+
+	dlg = mergeant_server_op_create_dialog (GTK_WINDOW (priv->window), GDA_SERVER_OPERATION_DROP_DB, 
+						gda_dict_get_connection (priv->dict), NULL,
+						_("Delete a database"),
+						_("Database destruction requires that first a database provider be selected."),
+						NULL, NULL);
+	gtk_widget_show (dlg);
+}
+
+static void
+on_tools_data_sources (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 	char *argv[2];
+	GError *error = NULL;
 
 	/* run gnome-database-properties dictig tool */
 	argv[0] = (char *) "gnome-database-properties";
         argv[1] = NULL;
                                                                                   
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		       NULL, NULL, NULL, NULL);
+	if (!g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+			    NULL, NULL, NULL, &error)) {
+		GtkWidget *dlg;
+		dlg = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_MODAL,
+							  GTK_MESSAGE_ERROR,
+							  GTK_BUTTONS_CLOSE, 
+							  "<b>%s:</b>\n\n%s",
+							  _("Could not find the 'gnome-database-properties' program"),
+							  error && error->message ? error->message : _("No detail"));
+		if (error)
+			g_error_free (error);
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+	}
 }
 
 
@@ -178,6 +214,7 @@ on_sql_console (GtkAction *action, void *data)
 	GtkWidget *dlg, *sw, *console;
 	WorkspaceWindowPrivate *priv;
 	gchar *title;
+	GdaConnection *cnc;
 
 	priv = g_object_get_data (G_OBJECT (data), "Mergeant_WorkspaceWindowPrivate");
 
@@ -186,11 +223,12 @@ on_sql_console (GtkAction *action, void *data)
 					   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 	
-	if (gnome_db_server_get_user_name (priv->server)) 
-		title = g_strdup_printf ("%s@%s - Mergeant", gnome_db_server_get_user_name (priv->server),
-					 gnome_db_server_get_datasource (priv->server));
-	else 
-		title = g_strdup_printf ("%s - Mergeant", gnome_db_server_get_datasource (priv->server));
+	cnc = gda_dict_get_connection (priv->dict);
+	if (gda_connection_get_username (cnc))
+                title = g_strdup_printf ("%s@%s - Mergeant", gda_connection_get_username (cnc),
+                                         gda_connection_get_dsn (cnc));
+        else
+                title = g_strdup_printf ("%s - Mergeant", gda_connection_get_dsn (cnc));
 	
 	gtk_window_set_title (GTK_WINDOW (dlg), title);
 	g_free (title);
@@ -298,17 +336,19 @@ on_help_about (GtkAction *action, void *data)
 
 static GtkActionEntry ui_actions[] = {
 	{ "File", NULL, "_File", NULL, "File", NULL },
-	{ "FileNewWorkspace", GTK_STOCK_NEW, "_New", NULL, "Open a new workspace", G_CALLBACK (on_file_new_workspace) },
+	{ "FileNewWorkspace", GTK_STOCK_NEW, "_New workspace", NULL, "Open a new workspace", G_CALLBACK (on_file_new_workspace) },
 	{ "FileCopyWorkspace", GTK_STOCK_COPY, "_Copy of this Workspace", NULL, "Open a new workspace with the same connection parameters", G_CALLBACK (on_file_copy_workspace) },
 	{ "FileClose", GTK_STOCK_CLOSE, "_Close", NULL, "Close this window", G_CALLBACK (on_file_close) },
-	{ "FileSave", GTK_STOCK_SAVE, "_Save", NULL, "Save metadata", G_CALLBACK (on_file_save) },
+	{ "FileSave", GTK_STOCK_SAVE, "_Save now", NULL, "Save metadata", G_CALLBACK (on_file_save) },
 	{ "FileExit", GTK_STOCK_QUIT, "_Quit", NULL, "Exit the application", G_CALLBACK (on_file_exit) },
 	{ "Database", NULL, "_Database", NULL, "Database", NULL },
 	{ "DatabaseSync", GTK_STOCK_REFRESH, "_Sync", NULL, "Refresh metadata from database", G_CALLBACK (on_database_sync) },
 	{ "DatabaseBeginTransaction", NULL, "_Begin", NULL, "Start a new transaction", G_CALLBACK (on_database_begin) },
 	{ "DatabaseCommitTransaction", NULL, "_Commit", NULL, "Commit current transaction", G_CALLBACK (on_database_commit) },
 	{ "DatabaseRollbackTransaction", NULL, "_Rollback", NULL, "Discard current transaction", G_CALLBACK (on_database_rollback) },
-	{ "ToolsDataSources", GTK_STOCK_PREFERENCES, "_Datasources", NULL, "Manage datasources", G_CALLBACK (on_tools_data_sources) },
+	{ "DatabaseCreate", GTK_STOCK_NEW, "_Create a database", NULL, "Create a new database", G_CALLBACK (on_database_new) },
+	{ "DatabaseDelete", GTK_STOCK_DELETE, "_Delete a database", NULL, "Delete an existing database", G_CALLBACK (on_database_del) },
+	{ "ToolsDataSources", GTK_STOCK_PREFERENCES, "_Datasources", NULL, "Manage the list of declared datasources", G_CALLBACK (on_tools_data_sources) },
 	{ "SqlConsole", NULL, "_Console SQL", NULL, "Open an SQL console", G_CALLBACK (on_sql_console) },
 	{ "About", NULL, "_About", NULL, "About", NULL },
 	{ "HelpAbout", GTK_STOCK_ABOUT, "_About", NULL, "Show information about Mergeant", G_CALLBACK (on_help_about) }
@@ -327,6 +367,8 @@ static const gchar *ui_actions_info =
         "    </menu>"
         "    <menu name='Database' action='Database'>"
         "      <menuitem name='ToolsDataSources' action= 'ToolsDataSources'/>"
+        "      <menuitem name='DatabaseCreate' action= 'DatabaseCreate'/>"
+        "      <menuitem name='DatabaseDelete' action= 'DatabaseDelete'/>"
         "      <separator/>"
         "      <menuitem name='DatabaseSync' action= 'DatabaseSync'/>"
         "      <separator/>"
@@ -336,21 +378,22 @@ static const gchar *ui_actions_info =
         "      <separator/>"
         "      <menuitem name='SqlConsole' action='SqlConsole'/>"
         "    </menu>"
+	"    <placeholder name='PageExtension'/>"
         "    <menu name='About' action='About'>"
         "      <menuitem name='HelpAbout' action= 'HelpAbout'/>"
         "    </menu>"
         "  </menubar>"
         "  <toolbar  name='ToolBar'>"
-        "    <toolitem action='DatabaseSync'/>"
-        "    <toolitem action='FileClose'/>"
+        "    <toolitem action='ToolsDataSources'/>"
+	"    <separator/>"
         "  </toolbar>"
         "</ui>";
 
+
 static void
-stop_dbms_update_cb (GtkWidget *dialog, gint response, GnomeDbDict *dict)
+stop_dbms_update_cb (GtkWidget *dialog, gint response, GdaDict *dict)
 {
-	gnome_db_server_stop_update_dbms_data (gnome_db_dict_get_server (dict));
-	gtk_widget_destroy (dialog);
+	gda_dict_stop_update_dbms_data (dict);
 }
 
 static void
@@ -370,21 +413,20 @@ perform_dbms_update (WorkspaceWindowPrivate *priv)
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), view, TRUE, TRUE, 6);
 	gtk_widget_show (dialog);
 	
-	/* update types, functions and aggregates */
 	error = NULL;
-	if (!gnome_db_server_update_dbms_data (priv->server, &error)) {
-		if (error->code != GNOME_DB_SERVER_META_DATA_UPDATE_USER_STOPPED) {
-			gnome_db_show_error (_("Error updating server metadata:\n%s"), error->message);
-			g_error_free (error);
+	if (!gda_dict_update_dbms_data (priv->dict, 0, NULL, &error)) {
+		if (error->code != GDA_DICT_META_DATA_UPDATE_USER_STOPPED) {
+			GtkWidget *msg;
+
+                        msg = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                                      GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                                      _("Error updating Server metadata:\n%s\n"), error->message);
+                        gtk_dialog_run (GTK_DIALOG (msg));
+                        gtk_widget_destroy (msg);
 		}
-	}
-	
-	/* update database */
-	error = NULL;
-	if (!gnome_db_database_update_dbms_data (gnome_db_dict_get_database (priv->dict), &error)) {
-		gnome_db_show_error (_("Error updating database metadata:\n%s"), error->message);
 		g_error_free (error);
-	}
+	}	
 	
 	gtk_widget_destroy (dialog);
 }
@@ -400,11 +442,12 @@ quit_if_no_other_workspace (void)
 }
 
 static void create_new_workspace_window_no_dict (WorkspaceOptions *options);
-static void create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean force_dbms_update, WorkspaceOptions *options);
+static void create_new_workspace_window_with_dict (GdaDict *dict, gboolean force_dbms_update, WorkspaceOptions *options);
 
 /**
  * create_new_workspace_window
- * @dict: a #GnomeDbDict object, or %NULL
+ * @dict: a #GdaDict object, or %NULL
+ * @ui:
  *
  * Creates a new workspace window. If @dict is not, %NULL
  * then it is reused for the new workspace.
@@ -412,11 +455,11 @@ static void create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean f
  * Returns:
  */
 void
-create_new_workspace_window (GnomeDbDict *dict)
+create_new_workspace_window (GdaDict *dict)
 {
 	if (dict) {
-		if (! IS_GNOME_DB_DICT (dict)) {
-			g_warning ("dict is not a GnomeDbDict object, cancelling workspace window creation");
+		if (! GDA_IS_DICT (dict)) {
+			g_warning ("dict is not a GdaDict object, cancelling workspace window creation");
 			quit_if_no_other_workspace ();
 			return;
 		}
@@ -424,12 +467,13 @@ create_new_workspace_window (GnomeDbDict *dict)
 		create_new_workspace_window_with_dict (dict, FALSE, NULL);
 	}
 	else
-		create_new_workspace_window_no_dict (NULL);	
+		create_new_workspace_window_no_dict (NULL);
 }
 
 /**
  * create_new_workspace_window_spec
  * @options: 
+ * @ui:
  *
  * Same as create_new_workspace_window but with specific options
  */
@@ -440,7 +484,7 @@ create_new_workspace_window_spec (WorkspaceOptions *options)
 }
 
 static void 
-create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean force_dbms_update, WorkspaceOptions *options)
+create_new_workspace_window_with_dict (GdaDict *dict, gboolean force_dbms_update, WorkspaceOptions *options)
 {
 	WorkspaceWindowPrivate *priv;
 	GtkWidget *table;
@@ -448,21 +492,22 @@ create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean force_dbms_up
 	GtkActionGroup *actions;
         GtkUIManager *ui;
 	GtkWidget *toolbar, *menubar;
+	GdaConnection *cnc;
 
 	/* here we MUST have a valid dictionary */
-	g_assert (dict && IS_GNOME_DB_DICT (dict));
+	g_assert (dict && GDA_IS_DICT (dict));
 	g_object_ref (dict);
 
 	/* create the private structure */
 	priv = g_new0 (WorkspaceWindowPrivate, 1);
 	priv->dict = dict;
-	priv->server = gnome_db_dict_get_server (priv->dict);
 	opened_workspaces = g_list_prepend (opened_workspaces, priv);
 
-	g_object_set (G_OBJECT (priv->server), "with_functions", TRUE, NULL);
+	gda_dict_extend_with_functions (priv->dict);
+
 	if (force_dbms_update) {
 		perform_dbms_update (priv);
-		gnome_db_dict_save_xml (dict, NULL);
+		gda_dict_save (dict, NULL);
 	}
 
 	/* create the window */
@@ -472,35 +517,22 @@ create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean force_dbms_up
 				priv, (GDestroyNotify) destroy_private_data_cb);
 	gtk_window_set_icon_from_file (GTK_WINDOW (priv->window), MERGEANT_PIXMAPDIR "/mergeant.png", NULL);
 
-	if (gnome_db_server_get_user_name (priv->server)) 
-		title = g_strdup_printf ("%s@%s - Mergeant", gnome_db_server_get_user_name (priv->server),
-					 gnome_db_server_get_datasource (priv->server));
-	else 
-		title = g_strdup_printf ("%s - Mergeant", gnome_db_server_get_datasource (priv->server));
+	cnc = gda_dict_get_connection (priv->dict);
+        if (gda_connection_get_username (cnc))
+                title = g_strdup_printf ("%s@%s - Mergeant", gda_connection_get_username (cnc),
+                                         gda_connection_get_dsn (cnc));
+        else
+                title = g_strdup_printf ("%s - Mergeant", gda_connection_get_dsn (cnc));
 	
 	gtk_window_set_title (GTK_WINDOW (priv->window), title);
 	g_free (title);
-	gtk_widget_set_size_request (priv->window, 850, 700);
-
-	/* create the workspace */
+	gtk_widget_set_size_request (priv->window, 900, 700);
 	table = gtk_table_new (4, 1, FALSE);
 
-	priv->workspace = workspace_new (priv->server);
-	gtk_widget_show (priv->workspace);
-	gtk_table_attach (GTK_TABLE (table), priv->workspace, 0, 1, 2, 3,
-			  GTK_SHRINK | GTK_FILL | GTK_EXPAND,
-			  GTK_SHRINK | GTK_FILL | GTK_EXPAND, 6, 6);
-
-	priv->status_bar = gtk_statusbar_new ();
-	gtk_widget_show (priv->status_bar);
-	gtk_table_attach (GTK_TABLE (table), priv->status_bar, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_container_add (GTK_CONTAINER (priv->window), table);
-	gtk_widget_show (table);
-
 	/* add the menu bar/toolbar */
 	actions = gtk_action_group_new ("Actions");
-	gtk_action_group_add_actions (actions, ui_actions, G_N_ELEMENTS (ui_actions), priv->window);
-	
+	gtk_action_group_add_actions (actions, ui_actions, G_N_ELEMENTS (ui_actions), priv);
+
 	ui = gtk_ui_manager_new ();
         gtk_ui_manager_insert_action_group (ui, actions, 0);
         gtk_ui_manager_add_ui_from_string (ui, ui_actions_info, -1, NULL);
@@ -513,6 +545,19 @@ create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean force_dbms_up
 	gtk_table_attach (GTK_TABLE (table), toolbar, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_widget_show (toolbar);
 
+	/* create the workspace */
+	priv->workspace = workspace_new (priv->dict, ui);
+	gtk_widget_show (priv->workspace);
+	gtk_table_attach (GTK_TABLE (table), priv->workspace, 0, 1, 2, 3,
+			  GTK_SHRINK | GTK_FILL | GTK_EXPAND,
+			  GTK_SHRINK | GTK_FILL | GTK_EXPAND, 6, 6);
+
+	priv->status_bar = gtk_statusbar_new ();
+	gtk_widget_show (priv->status_bar);
+	gtk_table_attach (GTK_TABLE (table), priv->status_bar, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_container_add (GTK_CONTAINER (priv->window), table);
+	gtk_widget_show (table);
+
 	/* display the window */
 	gtk_widget_show (priv->window);
 
@@ -544,16 +589,22 @@ create_new_workspace_window_with_dict (GnomeDbDict *dict, gboolean force_dbms_up
 
 /* structure holding data used during the threads usage */
 typedef struct {
-	GdaThreader  *threader;
-	GnomeDbDict  *dict;
-	gchar        *filename;
-	GError      **error;
-	gboolean      func_retval;
-	gboolean      force_dbms_update;
+	GdaThreader   *threader;
+	GdaDict       *dict;
+	gchar         *filename;
+	GError       **error;
 
-	GtkWidget    *label;
-	gint          timeout_id;
-	GtkWidget    *bar; /* pulse bar */
+	gchar         *dsn;
+	gchar         *username;
+	gchar         *password;
+	GdaConnection *cnc;
+
+	gboolean       func_retval;
+	gboolean       force_dbms_update;
+
+	GtkWidget     *label;
+	gint           timeout_id;
+	GtkWidget     *bar; /* pulse bar */
 } ThreadData;
 
 static void
@@ -563,10 +614,17 @@ thread_data_free (ThreadData *thdata)
 		g_object_unref (thdata->threader);
 	if (thdata->dict)
 		g_object_unref (thdata->dict);
-	if (thdata->filename)
-		g_free (thdata->filename);
-	if (thdata->error)
+	g_free (thdata->filename);
+	if (thdata->error) {
+		if (*(thdata->error))
+			g_error_free (*(thdata->error));
 		g_free (thdata->error);
+	}
+	g_free (thdata->dsn);
+	g_free (thdata->username);
+	g_free (thdata->password);
+	if (thdata->cnc)
+		g_object_unref (thdata->cnc);
 	g_free (thdata);
 }
 
@@ -586,10 +644,16 @@ update_progress_bar (GtkProgressBar *bar)
 static void
 thread_open_connect (ThreadData *thdata)
 {
-	GnomeDbServer *server;
+	GdaConnection *cnc;
+	static GdaClient *gda_client = NULL;	
 
-	server = gnome_db_dict_get_server (thdata->dict);
-	thdata->func_retval = gnome_db_server_open_connect (server, thdata->error);
+	if (!gda_client)
+		gda_client = gda_client_new ();
+	cnc = gda_dict_get_connection (thdata->dict);
+	g_assert (!cnc);
+	cnc = gda_client_open_connection (gda_client, thdata->dsn, thdata->username, thdata->password, 
+					  0, thdata->error);
+	thdata->cnc = cnc;
 }
 
 /* 
@@ -601,17 +665,32 @@ thread_open_connect_finished (GdaThreader *thread, guint job_id, ThreadData *thd
 	g_source_remove (thdata->timeout_id);
 	thdata->timeout_id = 0;
 
-	if (thdata->func_retval == 0) {
+	if (!thdata->cnc) {
 		GError *error = *(thdata->error);
 		/* gdk_threads_enter (); */
-		gnome_db_show_error (error->message);
+
+		GtkWidget *msg;
+
+		msg = gtk_message_dialog_new (NULL,
+					      GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+					      _("Error Opening connection:\n%s\n"), 
+					      error && error->message ? error->message : _("No detail"));
+		gtk_dialog_run (GTK_DIALOG (msg));
+		gtk_widget_destroy (msg);
+
 		/* gdk_threads_leave (); */
-		g_error_free (error);
+		if (error)
+			g_error_free (error);
 		quit_if_no_other_workspace ();
 	}
 	else {
 		WorkspaceOptions *options;
 
+		gda_dict_set_connection (thdata->dict, thdata->cnc);
+		g_object_unref (thdata->cnc);
+		thdata->cnc = NULL;
+
 		options = g_object_get_data (G_OBJECT (dsn_dialog), "options");
 		/* gdk_threads_enter (); */
 		create_new_workspace_window_with_dict (thdata->dict, thdata->force_dbms_update, options);
@@ -636,7 +715,7 @@ open_connection (ThreadData *thdata)
 	gtk_label_set_markup (GTK_LABEL (thdata->label), str);
 	g_free (str);
 	
-	thdata->timeout_id = g_timeout_add (50, (GSourceFunc) update_progress_bar, thdata->bar);
+	thdata->timeout_id = g_timeout_add (PROGRESS_BAR_TIMEOUT, (GSourceFunc) update_progress_bar, thdata->bar);
 	
 	gda_threader_start_thread (thdata->threader, (GThreadFunc) thread_open_connect, thdata, 
 				   (GdaThreaderFunc) thread_open_connect_finished, NULL, NULL);
@@ -648,7 +727,7 @@ open_connection (ThreadData *thdata)
 static void
 thread_load_dict (ThreadData *thdata)
 {
-	thdata->func_retval = gnome_db_dict_load_xml_file (thdata->dict, thdata->filename, thdata->error);
+	thdata->func_retval = gda_dict_load_xml_file (thdata->dict, thdata->filename, thdata->error);
 }
 
 static void
@@ -680,8 +759,8 @@ thread_load_dict_finished (GdaThreader *thread, guint job_id, ThreadData *thdata
 		GError *error = *(thdata->error);
 		thdata->force_dbms_update = TRUE;
 
-		if ((error->domain != GNOME_DB_DICT_ERROR) || 
-		    (error->code != GNOME_DB_DICT_LOAD_FILE_NOT_EXIST_ERROR)) {
+		if ((error->domain != GDA_DICT_ERROR) || 
+		    (error->code != GDA_DICT_LOAD_FILE_NOT_EXIST_ERROR)) {
 			GtkWidget *dlg;
 			gchar *msg;
 			
@@ -722,7 +801,7 @@ static void
 create_new_workspace_window_no_dict_choose_cb (GtkDialog *dialog, gint response, gpointer data)
 {
 	WorkspaceWindowPrivate *priv;
-	GnomeDbDict *workdict = NULL;
+	GdaDict *workdict = NULL;
 	const gchar *dsn;
 	GList *list = opened_workspaces;
 	gchar *username, *password;
@@ -736,10 +815,10 @@ create_new_workspace_window_no_dict_choose_cb (GtkDialog *dialog, gint response,
 	}
 
 	/* User has selected a data source */
-	username = gnome_db_login_dialog_get_username (GNOME_DB_LOGIN_DIALOG (dialog));
+	username = (gchar *) gnome_db_login_dialog_get_username (GNOME_DB_LOGIN_DIALOG (dialog));
 	if (username)
 		username = g_strdup (username);
-	password = gnome_db_login_dialog_get_password (GNOME_DB_LOGIN_DIALOG (dialog));
+	password = (gchar *) gnome_db_login_dialog_get_password (GNOME_DB_LOGIN_DIALOG (dialog));
 	if (password)
 		password = g_strdup (password);
 		
@@ -748,7 +827,7 @@ create_new_workspace_window_no_dict_choose_cb (GtkDialog *dialog, gint response,
 	while (list && !workdict) {
 		gchar *dsn2;
 		priv = (WorkspaceWindowPrivate *) list->data;
-		dsn2 = gnome_db_server_get_datasource (priv->server);
+		dsn2 = (gchar *) gda_connection_get_dsn (gda_dict_get_connection (priv->dict));
 		if (!strcmp (dsn, dsn2)) 
 			workdict = priv->dict;
 		g_free (dsn2);
@@ -769,23 +848,23 @@ create_new_workspace_window_no_dict_choose_cb (GtkDialog *dialog, gint response,
 	else {
 		gchar *filename = NULL;
 		ThreadData *thdata;
-		GnomeDbServer *server = NULL;
 		GdaThreader *threader;
 		GtkWidget *wid;		
 		gchar *str;
 
-		workdict = (GnomeDbDict *) gnome_db_dict_new ();
-		server = gnome_db_dict_get_server (workdict);
+		workdict = (GdaDict *) gda_dict_new ();
 		
-		gnome_db_server_set_datasource (server, dsn);
-		filename = gnome_db_dict_compute_xml_filename (workdict, dsn, NULL, NULL);
-		gnome_db_dict_set_xml_filename (workdict, filename);
-		gnome_db_server_set_user_name (server, username);
-		gnome_db_server_set_user_password (server, password);
+		filename = gda_dict_compute_xml_filename (workdict, dsn, NULL, NULL);
+		gda_dict_set_xml_filename (workdict, filename);
 
 		/* structure passed during the multi-thread */
 		thdata = g_new0 (ThreadData, 1);
 		thdata->dict = workdict;
+		thdata->dsn = g_strdup (dsn);
+		thdata->username = username;
+		username = NULL;
+		thdata->password = password;
+		password = NULL;
 		thdata->func_retval = FALSE;
 		thdata->filename = filename;
 		thdata->error = g_new0 (GError *, 1);
@@ -806,7 +885,7 @@ create_new_workspace_window_no_dict_choose_cb (GtkDialog *dialog, gint response,
 		thdata->bar = wid;
 		gtk_widget_show (wid);
 		
-		thdata->timeout_id = g_timeout_add (50, (GSourceFunc) update_progress_bar, wid);
+		thdata->timeout_id = g_timeout_add (PROGRESS_BAR_TIMEOUT, (GSourceFunc) update_progress_bar, wid);
 
 		/* try loading dictionary */
 		threader = GDA_THREADER (gda_threader_new ());
@@ -862,6 +941,7 @@ create_new_workspace_window_no_dict (WorkspaceOptions *options)
 		gtk_widget_show (hbox);
 
 		wid = gtk_progress_bar_new ();
+		gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (wid), 0.02);
 		g_object_set_data (G_OBJECT (dsn_dialog), "info_bar", wid);
 		gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, TRUE, 0);
 		gtk_widget_show (wid);
